/**
 * @file kiran-license-wrapper.h
 * @brief license状态获取对象封装
 * @author songchuanfei <songchuanfei@kylinos.com.cn>
 * @copyright (c) 2020 KylinSec. All rights reserved.
 *
 */

/**
 *  @mainpage API wrapper for system license
 *
 *## Usage
 *
 *- 包含头文件
 *
 *```
 *    #include <kiran-license-wrapper.h>
 *```
 *- 链接时添加license-wrapper库，见 `pkg-config --cflags --libs license-wrapper`
 *
 *
 *## Example
 *   @include example.c
 */
#pragma once
#include <gio/gio.h>
#include <glib-object.h>

/**
 * 当前系统授权状态
 */
typedef enum
{
    LICENSE_STATUS_UNREGISTERED = 0,     /**< 试用期内 */
    LICENSE_STATUS_REGISTERED,           /**< 已授权，在服务期内 */
    LICENSE_STATUS_NOREQUIRED,           /**< 不需要授权 */
    LICENSE_STATUS_REGISTERED_EXPIRED,   /**< 授权过期 */
    LICENSE_STATUS_UNREGISTERED_EXPIRED, /**< 试用期过期 */
    LICENSE_STATUS_UNKNOWN               /**< 授权状态未知 */
} KiranLicenseStatus;

G_BEGIN_DECLS

#define KIRAN_LICENSE_TYPE_WRAPPER kiran_license_wrapper_get_type()

G_DECLARE_FINAL_TYPE(KiranLicenseWrapper, kiran_license_wrapper, KIRAN_LICENSE, WRAPPER, GObject)

/**
 * @brief 创建一个新的license封装对象
 * 
 *        不再使用时，应调用 g_object_unref() 来销毁对象.
 *        该对象有两个信号:
 *        - void status-ready(): 系统授权状态可读时发出
 *        - void status-changed(): 系统授权状态发生变化时发出
 * 
 *        当收到上述信号时可调用 kiran_license_wrapper_get_status() 读取系统当前授权状态
 * 
 * @return 返回创建的license封装对象
 */
KiranLicenseWrapper *kiran_license_wrapper_new(void);

/**
 * @brief 获取当前license信息是否已经就绪
 *
 *        只有该接口返回TRUE的情况下，通过 kiran_license_wrapper_get_status() 获取的状态才
 *        是系统的真实的授权状态
 *
 * @param manager 通过 kiran_license_wrapper_new() 创建的对象
 * @return 授权信息已就绪返回true，否则返回false
 */
gboolean kiran_license_wrapper_is_ready(KiranLicenseWrapper *manager);

/**
 * @brief 获取当前系统的授权状态
 * 
 *        如果当前尚未从DBus后台读取到授权状态(kiran_license_wrapper_is_ready() 返回 false)，
 *        该接口会返回 LICENSE_STATUS_UNKNOWN，同时会重新连接dbus读取授权信息，读取到授权信息后
 *        会发出 status-ready 信号。如果之前已读取授权状态，直接返回之前读取的授权状态。
 * 
 * @param manager 通过 kiran_license_wrapper_new() 创建的对象
 * @return 当前系统的授权状态 @see KiranLicenseStatus
 */
KiranLicenseStatus kiran_license_wrapper_get_status(KiranLicenseWrapper *manager);

/**
 * @brief 同步获取当前系统的授权状态
 *
 *        kiran_license_wrapper_get_status() 的同步版本.
 *        区别在于如果当前尚未从DBus后台读取到授权状态(kiran_license_wrapper_is_ready() 返回 false)，
 *        会通过同步调用DBus接口来获取信息，出错时将设置error参数
 *
 * @param self    通过 kiran_license_wrapper_new() 创建的对象
 * @param error   用于错误报告的GError，可以为NULL。error为NULL时，将无法判断是否成功。
 * @return 当前系统的授权状态，获取失败时返回 LICENSE_STATUS_UNKNOWN，同时填充error参数
 */
KiranLicenseStatus kiran_license_wrapper_get_status_sync(KiranLicenseWrapper *self,
                                                         GError **error);

G_END_DECLS
